@@ -378,7 +378,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   };
   libaddr=(unsigned int)status;
   flags|=MAP_FIXED;
-  
+
   /* Get the memory to store the library */
   ppnt = (struct elf_phdr *) &header[epnt->e_phoff];
   
@@ -397,7 +397,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	unsigned int map_size;
 	char * cpnt;
 	
-	status = (char *) _dl_mmap((char *) (libaddr  +
+	status = (char *) _dl_mmap((char *) ((piclib?libaddr:0)  +
 					     (ppnt->p_vaddr & 0xfffff000)),
 			  (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz, 
 			  LXFLAGS(ppnt->p_flags), 
@@ -412,25 +412,23 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	    return NULL;
 	};
 	
-	if(!piclib) status = 0;
-	
 	/* Pad the last page with zeroes. */
 	cpnt =(char *) (status + (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz);
 	while(((unsigned int) cpnt) & 0xfff) *cpnt++ = 0;
-	
+
 /* I am not quite sure if this is completely correct to do or not, but
    the basic way that we handle bss segments is that we mmap /dev/zero if
    there are any pages left over that are not mapped as part of the file */
 
 	map_size = (ppnt->p_vaddr + ppnt->p_filesz + 0xfff) & 0xfffff000;
 	if(map_size < ppnt->p_vaddr + ppnt->p_memsz)
-	  status = (char *) _dl_mmap((char *) map_size + libaddr, 
+	  status = (char *) _dl_mmap((char *) map_size + (piclib?libaddr:0), 
 			    ppnt->p_vaddr + ppnt->p_memsz - map_size,
 			    LXFLAGS(ppnt->p_flags),
 			    flags | MAP_ANONYMOUS, -1, 0);
       } else
 	status = (char *) _dl_mmap((char *) (ppnt->p_vaddr & 0xfffff000) + 
-				   libaddr, 
+				   (piclib?libaddr:0), 
 			  (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz, 
 			  LXFLAGS(ppnt->p_flags), 
 			  flags, infile, 
@@ -442,6 +440,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	_dl_close(infile);
 	return NULL;
       };
+
       /* if(libaddr == 0 && piclib) {
 	libaddr = (unsigned int) status;
 	flags |= MAP_FIXED;
@@ -452,9 +451,9 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   _dl_close(infile);
   
   /* For a non-PIC library, the addresses are all absolute */
-  if(!piclib) libaddr = 0;
-
-  dynamic_addr += (unsigned int) libaddr;
+  if(piclib) {
+    dynamic_addr += (unsigned int) libaddr;
+  }
 
  /* 
   * OK, the ELF library is now loaded into VM in the correct locations
@@ -489,7 +488,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   ppnt = (struct elf_phdr *) &header[epnt->e_phoff];
   for(i=0;i < epnt->e_phnum; i++, ppnt++){
     if(ppnt->p_type == PT_LOAD && !(ppnt->p_flags & PF_W))
-      _dl_mprotect((void *) (libaddr + (ppnt->p_vaddr & 0xfffff000)),
+      _dl_mprotect((void *) ((piclib?libaddr:0) + (ppnt->p_vaddr & 0xfffff000)),
 		   (ppnt->p_vaddr & 0xfff) + (unsigned int) ppnt->p_filesz,
 		   PROT_READ | PROT_WRITE | PROT_EXEC);
   }
